SystemOrganization addCategory: #'System-Tests-ChangeNotification'!

TestCase subclass: #SystemChangeTestRoot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tests-ChangeNotification'!

!SystemChangeTestRoot commentStamp: 'rw 4/5/2006 17:28' prior: 0!
The Root test class for the System Change Notification tests.!

SystemChangeTestRoot subclass: #ChangeHooksTest
	instanceVariableNames: 'previousChangeSet testsChangeSet capturedEvents generatedTestClass generatedTestClassX createdMethodName createdMethod doItExpression'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tests-ChangeNotification'!

!ChangeHooksTest commentStamp: 'rw 4/5/2006 17:14' prior: 0!
This class implements unit tests to verify that when the system changes, notification messages are sent around correctly.

Therefore the test messages make a system change, after registering to receive an event ater the change occured. In this event (sent immediately after the change), the actual assertions take place.

Note that the system changes are *really* made to the system, but in a change set that is created in the setUp method, while the previous one is restored in the tearDown method.!

!ChangeHooksTest methodsFor: 'Private' stamp: 'rw 8/1/2003 17:03'!
addSingleEvent: anEvent

	capturedEvents isEmpty ifFalse: [self assert: false].
	capturedEvents add: anEvent! !

!ChangeHooksTest methodsFor: 'Private' stamp: 'rw 7/11/2003 09:55'!
checkEvent: anEvent kind: changeKind item: item itemKind: itemKind 

	self assert: (anEvent perform: ('is' , changeKind) asSymbol).
	self assert: anEvent item = item.
	self assert: anEvent itemKind = itemKind! !

!ChangeHooksTest methodsFor: 'Private' stamp: 'rw 8/1/2003 17:01'!
checkForOnlySingleEvent

	self assert: capturedEvents size = 1! !

!ChangeHooksTest methodsFor: 'Events-Classes' stamp: 'rw 4/5/2006 17:15'!
classCommentedEvent: event 

	self addSingleEvent: event.
	self assert: generatedTestClass comment = self commentStringForTesting.
	self 
		checkEvent: event
		kind: #Commented
		item: generatedTestClass
		itemKind: AbstractEvent classKind! !

!ChangeHooksTest methodsFor: 'Events-Classes' stamp: 'StephaneDucasse 8/19/2010 22:25'!
classCreationEvent: event 

	| classCreated |
	self addSingleEvent: event.
	classCreated := Smalltalk globals classNamed: self newlyCreatedClassName.
	self assert: classCreated notNil.
	self 
		assert: ((Smalltalk globals organization 
				listAtCategoryNamed: #'System-Change Notification') 
					includes: self newlyCreatedClassName).
	self 
		checkEvent: event
		kind: #Added
		item: classCreated
		itemKind: AbstractEvent classKind! !

!ChangeHooksTest methodsFor: 'Events-Classes' stamp: 'rw 4/5/2006 17:15'!
classRecategorizedEvent: event 

	self addSingleEvent: event.
	self 
		checkEvent: event
		kind: #Recategorized
		item: generatedTestClass
		itemKind: AbstractEvent classKind.
	self assert: event oldCategory = #'System-Change Notification'! !

!ChangeHooksTest methodsFor: 'Events-Classes' stamp: 'rw 4/5/2006 17:15'!
classRedefinitionEvent: event 
	
	self addSingleEvent: event.
	self 
		checkEvent: event
		kind: #Modified
		item: generatedTestClass
		itemKind: AbstractEvent classKind.! !

!ChangeHooksTest methodsFor: 'Events-Classes' stamp: 'StephaneDucasse 8/19/2010 22:25'!
classRemovalEvent: event 
	"This event used to be sent efter the class was removed.
	This was changed, and therefore this test is useless currently."

	self addSingleEvent: event.
	self assert: (Smalltalk globals classNamed: self generatedTestClassName) isNil.
	self 
		checkEvent: event
		kind: #Removed
		item: self generatedTestClassName
		itemKind: AbstractEvent classKind! !

!ChangeHooksTest methodsFor: 'Events-Classes' stamp: 'StephaneDucasse 8/19/2010 22:30'!
classRenameEvent: event 

	| renamedClass |
	self addSingleEvent: event.
	renamedClass := Smalltalk globals classNamed: self renamedTestClassName.
	self assert: renamedClass notNil.
	self assert: (Smalltalk globals classNamed: self generatedTestClassName) isNil.
	self 
		checkEvent: event
		kind: #Renamed
		item: renamedClass
		itemKind: AbstractEvent classKind.
	self assert: event oldName = self generatedTestClassName! !

!ChangeHooksTest methodsFor: 'Events-Classes' stamp: 'rw 4/5/2006 17:16'!
classSuperChangedEvent: event 

	self addSingleEvent: event.
	self 
		checkEvent: event
		kind: #Modified
		item: generatedTestClass
		itemKind: AbstractEvent classKind.
	self assert: generatedTestClass superclass = Model! !

!ChangeHooksTest methodsFor: 'Private' stamp: 'rw 7/2/2003 18:07'!
commentStringForTesting

	^'Added this comment as part of the unit test in SystemChangeTest>>testClassCommentedBasicEvents. You should never see this, unless you are debugging the system somewhere in between the tests.'! !

!ChangeHooksTest methodsFor: 'Private-Generation' stamp: 'rw 4/4/2006 21:41'!
generateTestClass

	generatedTestClass := Object
				subclass: self generatedTestClassName
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: 'System-Change Notification'.! !

!ChangeHooksTest methodsFor: 'Private-Generation' stamp: 'rw 4/4/2006 21:41'!
generateTestClassX

	generatedTestClassX := Object
				subclass: self generatedTestClassNameX
				instanceVariableNames: 'x'
				classVariableNames: ''
				poolDictionaries: ''
				category: 'System-Change Notification'.! !

!ChangeHooksTest methodsFor: 'Private-Generation' stamp: 'rw 4/4/2006 22:10'!
generatedTestClassName


	^#'AutoGeneratedClassForTestingSystemChanges'! !

!ChangeHooksTest methodsFor: 'Private-Generation' stamp: 'rw 4/4/2006 22:10'!
generatedTestClassNameX

	^#'AutoGeneratedClassXForTestingSystemChanges'! !

!ChangeHooksTest methodsFor: 'Events-Instance Variables' stamp: 'rw 4/5/2006 17:16'!
instanceVariableCreationEvent: event

	self addSingleEvent: event.	
	self assert: event isModified.
	self assert: event item = generatedTestClass.
	self assert: event itemKind = AbstractEvent classKind.
	self assert: event areInstVarsModified.
	self deny: event isSuperclassModified.
	self deny: event areClassVarsModified.
	self deny: event areSharedPoolsModified.
	
! !

!ChangeHooksTest methodsFor: 'Events-Instance Variables' stamp: 'rw 4/5/2006 17:18'!
instanceVariableRemovedEvent: event

	self addSingleEvent: event.	
	self assert: event isModified.
	self assert: event item = generatedTestClassX.
	self assert: event itemKind = AbstractEvent classKind.
	self assert: event areInstVarsModified.
	self deny: event isSuperclassModified.
	self deny: event areClassVarsModified.
	self deny: event areSharedPoolsModified.
	
! !

!ChangeHooksTest methodsFor: 'Events-Methods' stamp: 'rw 4/5/2006 17:18'!
methodCreationEvent1: event 

	| methodCreated |
	self addSingleEvent: event.
	self shouldnt: [methodCreated := generatedTestClass >> createdMethodName]
		raise: Error.
	self 
		checkEvent: event
		kind: #Added
		item: methodCreated
		itemKind: AbstractEvent methodKind! !

!ChangeHooksTest methodsFor: 'Events-Methods' stamp: 'rw 4/5/2006 17:18'!
methodCreationEvent2: event 

	| methodCreated |
	self addSingleEvent: event.
	self shouldnt: [methodCreated := generatedTestClass >> createdMethodName]
		raise: Error.
	self 
		checkEvent: event
		kind: #Added
		item: methodCreated
		itemKind: AbstractEvent methodKind! !

!ChangeHooksTest methodsFor: 'Events-Expression' stamp: 'rw 4/5/2006 17:16'!
methodDoItEvent1: event 

	self addSingleEvent: event.
	self 
		checkEvent: event
		kind: #DoIt
		item: doItExpression
		itemKind: AbstractEvent expressionKind.
	self assert: event context isNil.! !

!ChangeHooksTest methodsFor: 'Events-Methods' stamp: 'rw 4/5/2006 17:18'!
methodRecategorizationEvent: event

	| methodCreated |
	self addSingleEvent: event.
	self shouldnt: [methodCreated := generatedTestClass >> createdMethodName]
		raise: Error.
	self assert: ((generatedTestClass organization categoryOfElement: createdMethodName) = #newCategory).
	self assert: event oldCategory = #testing.
	self 
		checkEvent: event
		kind: #Recategorized
		item: methodCreated
		itemKind: AbstractEvent methodKind.! !

!ChangeHooksTest methodsFor: 'Events-Methods' stamp: 'rw 4/5/2006 17:18'!
methodRemovedEvent1: event 

	self addSingleEvent: event.
	self should: [generatedTestClass >> createdMethodName] raise: Error.
	self 
		checkEvent: event
		kind: #Removed
		item: createdMethod
		itemKind: AbstractEvent methodKind.
	event itemClass = generatedTestClass.
	event itemMethod = createdMethodName.
	self assert: ((generatedTestClass organization categoryOfElement: createdMethodName) isNil).! !

!ChangeHooksTest methodsFor: 'Events-Methods' stamp: 'rw 4/5/2006 17:18'!
methodRemovedEvent2: event 

	self methodRemovedEvent1: event! !

!ChangeHooksTest methodsFor: 'Private-Generation' stamp: 'rw 4/4/2006 22:18'!
newlyCreatedClassName

	^#'AutoGeneratedClassWhileTestingSystemChanges'! !

!ChangeHooksTest methodsFor: 'Events-General' stamp: 'rw 8/1/2003 17:11'!
rememberEvent: event

	capturedEvents add: event! !

!ChangeHooksTest methodsFor: 'Private' stamp: 'StephaneDucasse 8/19/2010 22:32'!
removeGeneratedTestClasses
	"Remove all classes that were possibly generated during testing."
	
	| possiblyToRemove |
	possiblyToRemove := OrderedCollection
		with: self generatedTestClassName
		with: self generatedTestClassNameX
		with: self renamedTestClassName
		with: self newlyCreatedClassName.
	possiblyToRemove do: [:name | (Smalltalk globals hasClassNamed: name) ifTrue: [(Smalltalk globals at: name) removeFromSystemUnlogged]]! !

!ChangeHooksTest methodsFor: 'Private-Generation' stamp: 'rw 4/4/2006 22:10'!
renamedTestClassName


	^#'AutoRenamedClassForTestingSystemChanges'! !

!ChangeHooksTest methodsFor: 'Running' stamp: 'rw 4/4/2006 22:59'!
setUp

	previousChangeSet := ChangeSet current.
	testsChangeSet := ChangeSet new.
	ChangeSet newChanges: testsChangeSet.
	capturedEvents := OrderedCollection new.
	self generateTestClass.
	self generateTestClassX.
	super setUp! !

!ChangeHooksTest methodsFor: 'Events-General' stamp: 'rw 8/1/2003 16:41'!
shouldNotBeCalledEvent: anEvent
	"This event should not be called, so fail the test."

	self assert: false! !

!ChangeHooksTest methodsFor: 'Running' stamp: 'tg 6/11/2010 14:04'!
tearDown

	super tearDown.
	self removeGeneratedTestClasses.
	ChangeSet newChanges: previousChangeSet.
	ChangeSet removeChangeSet: testsChangeSet.
	previousChangeSet := nil.
	testsChangeSet := nil.
! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:19'!
testClassCommentedEvent

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classCommentedEvent:.
	generatedTestClass comment: self commentStringForTesting.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:19'!
testClassCreationEvent

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classCreationEvent:.
	Object 
		subclass: self newlyCreatedClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'System-Change Notification'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:19'!
testClassRecategorizedEvent1

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classRecategorizedEvent:.
	Object 
		subclass: generatedTestClass name
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Collections-Abstract'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:19'!
testClassRecategorizedEvent2

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classRecategorizedEvent:.
	generatedTestClass category: 'Collections-Abstract'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:19'!
testClassRedefinition

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classRedefinitionEvent:.
	self generateTestClass! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:20'!
testClassRemovalEvent
	"This event used to be sent efter the class was removed.
	This was changed, and therefore this test is useless currently."
	
	"Keep it, since I really want to check with the responsible for the ChangeSet,
	and it is very likely this will be reintroduced afterwards!!"

"	| createdClass |
	createdClass := self compileUniqueClass.
	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classRemovalEvent:.
	createdClass removeFromSystem.
	self checkForOnlySingleEvent
	
	"! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:20'!
testClassRenamedEvent

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classRenameEvent:.
	generatedTestClass rename: self renamedTestClassName.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Classes' stamp: 'rw 4/5/2006 17:20'!
testClassSuperChangedEvent

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #classSuperChangedEvent:.
	Model 
		subclass: generatedTestClass name
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'System-Change Notification'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Expression' stamp: 'rw 4/5/2006 17:20'!
testDoItEvent1

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #methodDoItEvent1:.
	doItExpression := '1 + 2'.
	Compiler evaluate: doItExpression logged: true.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Expression' stamp: 'rw 4/5/2006 17:20'!
testDoItEvent2

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #shouldNotBeCalledEvent:.
	doItExpression := '1 + 2'.
	Compiler evaluate: doItExpression logged: false.! !

!ChangeHooksTest methodsFor: 'Testing-Instance Variables' stamp: 'rw 4/5/2006 17:20'!
testInstanceVariableCreationEvent1

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #instanceVariableCreationEvent:.
	Object 
		subclass: self generatedTestClassName
		instanceVariableNames: 'x'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'System-Change Notification'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Instance Variables' stamp: 'rw 4/5/2006 17:17'!
testInstanceVariableCreationEvent2

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #instanceVariableCreationEvent:.
	generatedTestClass addInstVarName: 'x'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Instance Variables' stamp: 'rw 4/5/2006 17:18'!
testInstanceVariableRemovedEvent1

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #instanceVariableRemovedEvent:.
	Object 
		subclass: generatedTestClassX name
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'System-Change Notification'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Instance Variables' stamp: 'rw 4/5/2006 17:17'!
testInstanceVariableRemovedEvent2

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #instanceVariableRemovedEvent:.
	generatedTestClassX removeInstVarName: 'x'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Instance Variables' stamp: 'rw 4/5/2006 17:21'!
testInstanceVariableRenamedSilently

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #shouldNotBeCalledEvent:.
	generatedTestClassX renameSilentlyInstVar: 'x' to: 'y'! !

!ChangeHooksTest methodsFor: 'Testing-Methods' stamp: 'rw 4/5/2006 17:21'!
testMethodCreationEvent1

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #methodCreationEvent1:.
	createdMethodName := #testCreation.
	generatedTestClass compile: createdMethodName , '	^1'.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Methods' stamp: 'rw 4/5/2006 17:21'!
testMethodCreationEvent2

	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #methodCreationEvent2:.
	createdMethodName := #testCreation.
	generatedTestClass compile: createdMethodName , '	^1' classified: #testing.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Methods' stamp: 'rw 4/5/2006 17:21'!
testMethodRecategorizationEvent

	createdMethodName := #testCreation.
	generatedTestClass compile: createdMethodName , '	^1' classified: #testing.
	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #methodRecategorizationEvent:.
	generatedTestClass organization 
		classify: createdMethodName
		under: #newCategory
		suppressIfDefault: false.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Methods' stamp: 'rw 4/5/2006 17:21'!
testMethodRemovedEvent1

	createdMethodName := #testCreation.
	generatedTestClass compile: createdMethodName , '	^1'.
	createdMethod := generatedTestClass >> createdMethodName.
	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #methodRemovedEvent1:.
	generatedTestClass removeSelector: createdMethodName.
	self checkForOnlySingleEvent! !

!ChangeHooksTest methodsFor: 'Testing-Methods' stamp: 'rw 4/5/2006 17:21'!
testMethodRemovedEvent2

	createdMethodName := #testCreation.
	generatedTestClass compile: createdMethodName , '	^1'.
	createdMethod := generatedTestClass >> createdMethodName.
	self systemChangeNotifier notify: self
		ofAllSystemChangesUsing: #methodRemovedEvent2:.
	Smalltalk 
		removeSelector: (Array with: generatedTestClass name with: createdMethodName).
	self checkForOnlySingleEvent! !

SystemChangeTestRoot subclass: #SystemChangeErrorHandlingTest
	instanceVariableNames: 'capturedEvents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tests-ChangeNotification'!

!SystemChangeErrorHandlingTest commentStamp: 'rw 4/3/2006 17:21' prior: 0!
This class tests the error handing of the notification mechanism to ensure that one client that receives a system change cannot lock up the complete system."!

!SystemChangeErrorHandlingTest methodsFor: 'Event Notifications' stamp: 'rw 4/3/2006 17:57'!
handleEventWithError: event

	self error: 'Example of event handling code that throws an error.'! !

!SystemChangeErrorHandlingTest methodsFor: 'Event Notifications' stamp: 'rw 4/3/2006 18:07'!
handleEventWithHalt: event

	self halt: 'Example of event handling code that contains a halt.'! !

!SystemChangeErrorHandlingTest methodsFor: 'Running' stamp: 'rw 4/3/2006 17:51'!
setUp

	super setUp.
	capturedEvents := OrderedCollection new! !

!SystemChangeErrorHandlingTest methodsFor: 'Event Notifications' stamp: 'rw 4/3/2006 17:57'!
storeEvent1: anEvent

	capturedEvents add: anEvent! !

!SystemChangeErrorHandlingTest methodsFor: 'Event Notifications' stamp: 'rw 4/3/2006 17:57'!
storeEvent2: anEvent

	capturedEvents add: anEvent! !

!SystemChangeErrorHandlingTest methodsFor: 'Event Notifications' stamp: 'rw 4/3/2006 17:57'!
storeEvent3: anEvent

	capturedEvents add: anEvent! !

!SystemChangeErrorHandlingTest methodsFor: 'Running' stamp: 'rw 4/3/2006 17:56'!
tearDown

	capturedEvents := nil.
	super tearDown! !

!SystemChangeErrorHandlingTest methodsFor: 'Testing' stamp: 'rw 4/5/2006 17:24'!
testErrorOperation

	| notifier wasCaptured |
	notifier := self systemChangeNotifier.
	wasCaptured := false.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #storeEvent1:.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #storeEvent2:.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #handleEventWithError:.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #storeEvent3:.
	[notifier classAdded: self class inCategory: #FooCat] on: Error do: [:exc |
		wasCaptured := true.
		self assert: (capturedEvents size = 3)].
	self assert: wasCaptured.! !

!SystemChangeErrorHandlingTest methodsFor: 'Testing' stamp: 'rw 4/5/2006 17:24'!
testHaltOperation
	
	| notifier wasCaptured |
	notifier := self systemChangeNotifier.
	wasCaptured := false.
	notifier notify: self ofAllSystemChangesUsing: #storeEvent1:.
	notifier notify: self ofAllSystemChangesUsing: #storeEvent2:.
	notifier notify: self ofAllSystemChangesUsing: #handleEventWithHalt:.
	notifier notify: self ofAllSystemChangesUsing: #storeEvent3:.
	[notifier classAdded: self class inCategory: #FooCat] on: Halt do: [:exc |
		wasCaptured := true.
		self assert: (capturedEvents size = 3)].
	self assert: wasCaptured.! !

!SystemChangeErrorHandlingTest methodsFor: 'Testing' stamp: 'rw 4/5/2006 17:24'!
testUnhandledEventOperation

	| notifier wasCaptured |
	notifier := self systemChangeNotifier.
	wasCaptured := false.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #storeEvent1:.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #storeEvent2:.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #zork:.
	notifier notify: self ofSystemChangesOfItem: #class change: #Added using: #storeEvent3:.
	[notifier classAdded: self class inCategory: #FooCat] on: MessageNotUnderstood do: [:exc |
		wasCaptured := true.
		self assert: (capturedEvents size = 3)].
	self assert: wasCaptured.! !

SystemChangeTestRoot subclass: #SystemChangeNotifierTest
	instanceVariableNames: 'capturedEvent notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tests-ChangeNotification'!

!SystemChangeNotifierTest commentStamp: 'rw 4/3/2006 17:19' prior: 0!
A SystemChangeNotifierTest is a test class that tests whether the triggering of changes indeed results in the intended changes to be sent to registered object. The basic mechanism for each test is fairly simple:
	- register the receiver as the one to get the change notifier.
	- manually trigger a change (so the system is not polluted just to see whether we get the needed event).
	- the method #event: is invoked and remembers the change event.
	- the change event is checked to see whether it was the intended one.

Instance Variables
	capturedEvent:		Remembers the captured event!

!SystemChangeNotifierTest methodsFor: 'Private' stamp: 'rw 4/3/2006 16:34'!
capturedEvent: eventOrNil
	"Remember the event being sent."

	capturedEvent := eventOrNil! !

!SystemChangeNotifierTest methodsFor: 'Private' stamp: 'rw 4/3/2006 16:36'!
checkEventForClass: aClass category: cat change: changeKind 

	self assert: (capturedEvent perform: ('is' , changeKind) asSymbol).
	self assert: capturedEvent item = aClass.
	self assert: capturedEvent itemKind = AbstractEvent classKind.
	self assert: capturedEvent itemClass = aClass.
	self assert: capturedEvent itemCategory = cat! !

!SystemChangeNotifierTest methodsFor: 'Private' stamp: 'rw 4/3/2006 16:43'!
checkEventForMethod: aMethod protocol: prot change: changeKind 

	self assert: (capturedEvent perform: ('is' , changeKind) asSymbol).
	self assert: capturedEvent item = aMethod.
	self assert: capturedEvent itemKind = AbstractEvent methodKind.
	self assert: capturedEvent itemClass = self class.
	self assert: capturedEvent itemMethod = aMethod.
	self assert: capturedEvent itemProtocol = prot! !

!SystemChangeNotifierTest methodsFor: 'Private' stamp: 'rw 4/3/2006 16:43'!
checkEventForMethod: aMethod protocol: prot change: changeKind oldMethod: oldMethod

	self checkEventForMethod: aMethod protocol: prot change: changeKind.
	self assert: capturedEvent oldItem == oldMethod
	! !

!SystemChangeNotifierTest methodsFor: 'Event Notifications' stamp: 'rw 4/3/2006 17:50'!
event: event
	"The notification message being sent to me when an event is captured. Remember it."

"	capturedEvent isNil ifTrue: [	self capturedEvent: event] ifFalse: [self assert: false]"

	self capturedEvent: event! !

!SystemChangeNotifierTest methodsFor: 'Running' stamp: 'rw 10/19/2006 17:21'!
setUp

	super setUp.
	notifier := SystemChangeNotifier createInstance.! !

!SystemChangeNotifierTest methodsFor: 'Private' stamp: 'rw 10/19/2006 17:23'!
systemChangeNotifier
	"The notifier to use. Do not use the one in the system so that the fake events triggered in the tests perturb clients of the system's change notifier (e.g. the changes file then shows fake entries)."

	^notifier! !

!SystemChangeNotifierTest methodsFor: 'Running' stamp: 'rw 10/19/2006 17:23'!
tearDown

	super tearDown.
	self capturedEvent: nil.
	notifier releaseAll.
	notifier := nil! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:24'!
testClassAddedEvent

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier classAdded: self class inCategory: #FooCat.
	self
		checkEventForClass: self class
		category: #FooCat
		change: #Added! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:24'!
testClassAddedEvent2

	self systemChangeNotifier notify: self ofSystemChangesOfItem: #class change: #Added using: #event:.
	self systemChangeNotifier classAdded: self class inCategory: #FooCat.
	self
		checkEventForClass: self class
		category: #FooCat
		change: #Added! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:24'!
testClassCommentedEvent

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier classCommented: self class inCategory: #FooCat.
	self
		checkEventForClass: self class
		category: #FooCat
		change: #Commented! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:24'!
testClassRecategorizedEvent

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier 
		class: self class
		recategorizedFrom: #FooCat
		to: #FooBar.
	self
		checkEventForClass: self class
		category: #FooBar
		change: #Recategorized.
	self assert: capturedEvent oldCategory = #FooCat! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:24'!
testClassRemovedEvent

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier classRemoved: self class fromCategory: #FooCat.
	self
		checkEventForClass: self class
		category: #FooCat
		change: #Removed! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 10/19/2006 17:24'!
testClassRenamedEvent
	"self run: #testClassRenamedEvent"

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier 
		classRenamed: self class
		from: #OldFooClass
		to: #NewFooClass
		inCategory: #FooCat.
	self
		checkEventForClass: self class
		category: #FooCat
		change: #Renamed.
"	self assert: capturedEvent oldName = #OldFooClass.
	self assert: capturedEvent newName = #NewFooClass"! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:25'!
testDoItEvent

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier 
		evaluated: '1 + 2'
		context: self.
	self assert: capturedEvent isDoIt.
	self assert: capturedEvent item = '1 + 2'.
	self assert: capturedEvent itemKind = AbstractEvent expressionKind.
	self assert: capturedEvent itemClass = nil.
	self assert: capturedEvent itemMethod = nil.
	self assert: capturedEvent itemProtocol = nil.
	self assert: capturedEvent itemExpression = '1 + 2'.
	self assert: capturedEvent context = self.! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:25'!
testMethodAddedEvent1

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier 
		methodAdded: self class >> #testMethodAddedEvent1
		selector: #testMethodAddedEvent1
		inProtocol: #FooCat
		class: self class.
	self 
		checkEventForMethod: self class >> #testMethodAddedEvent1
		protocol: #FooCat
		change: #Added! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:25'!
testMethodAddedEvent2

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier 
		methodAdded: self class >> #testMethodAddedEvent1
		selector: #testMethodAddedEvent1
		inClass: self class.
	self 
		checkEventForMethod: self class >> #testMethodAddedEvent1
		protocol: nil
		change: #Added! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:25'!
testMethodAddedEvent3

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier 
		methodChangedFrom: self class >> #testMethodAddedEvent1
		to: self class >> #testMethodAddedEvent2
		selector: #testMethodAddedEvent2
		inClass: self class.
	self 
		checkEventForMethod: self class >> #testMethodAddedEvent2
		protocol: nil
		change: #Modified
		oldMethod: self class >> #testMethodAddedEvent1.! !

!SystemChangeNotifierTest methodsFor: 'Testing-system triggers' stamp: 'rw 4/5/2006 17:25'!
testMethodRemovedEvent

	self systemChangeNotifier notify: self ofAllSystemChangesUsing: #event:.
	self systemChangeNotifier 
		methodRemoved: self class>> #testMethodRemovedEvent
		selector: #testMethodRemovedEvent
		inProtocol: #FooCat
		class: self class.
	self
		checkEventForMethod: self class>> #testMethodRemovedEvent
		protocol: #FooCat
		change: #Removed.! !

!SystemChangeTestRoot methodsFor: 'Private' stamp: 'rw 4/3/2006 17:48'!
systemChangeNotifier
	"The notifier to use. Use the one for the system."

	^SystemChangeNotifier uniqueInstance! !

!SystemChangeTestRoot methodsFor: 'Running' stamp: 'rw 4/3/2006 17:59'!
tearDown

	self unhook.
	super tearDown! !

!SystemChangeTestRoot methodsFor: 'Running' stamp: 'rw 4/3/2006 17:23'!
unhook

	self systemChangeNotifier noMoreNotificationsFor: self! !
